import React, { useEffect, useState } from 'react';
import MonacoEditor from "@monaco-editor/react";
import { Button, Container, Panel } from '@playcanvas/pcui/react';

const ControlPanel = (props: any) => {
    const [state, setState] = useState({
        showParameters: false,
        showCode: true,
        collapsed: window.top.innerWidth < 601
    });
    const onClickParametersTab = () => {
        if (document.getElementById('paramButton').classList.contains('selected')) {
            return;
        }
        setState({
            showParameters: true,
            showCode: false,
            collapsed: false
        });
        document.getElementById('paramButton').classList.toggle('selected');
        document.getElementById('codeButton').classList.toggle('selected');
        const controls = document.getElementById('controlPanel-controls');
        controls.classList.remove('pcui-hidden');
    };
    const onClickCodeTab = () => {
        if (document.getElementById('codeButton').classList.contains('selected')) {
            return;
        }
        setState({
            showParameters: false,
            showCode: true,
            collapsed: false
        });
        document.getElementById('paramButton').classList.toggle('selected');
        document.getElementById('codeButton').classList.toggle('selected');
        const controls = document.getElementById('controlPanel-controls');
        controls.classList.add('pcui-hidden');
    };

    useEffect(() => {
        if (window.top.innerWidth < 601) {
            // @ts-ignore
            document.getElementById('controlPanel-controls').ui.hidden = true;
        }
        if (window.top.location.hash.indexOf('#/iframe') === 0) {
            // @ts-ignore
            document.getElementById('controlPanel').ui.hidden = true;
        }
    });

    return <Panel id='controlPanel' class={[window.top.innerWidth > 600 && !props.controls ? 'empty' : 'null', window.top.innerWidth < 601 ? 'mobile' : null]} resizable='top' headerText={window.top.innerWidth < 601 ? (props.controls ? 'CODE & CONTROLS' : 'CODE') : 'CONTROLS'} collapsible={true} collapsed={state.collapsed}>
        { window.top.innerWidth < 601 && props.controls && <Container id= 'controlPanel-tabs' class='tabs-container'>
            <Button text='CODE' id='codeButton' class={state.showCode ? 'selected' : null} onClick={onClickCodeTab}/>
            <Button text='PARAMETERS' class={state.showParameters ? 'selected' : null} id='paramButton' onClick={onClickParametersTab} />
        </Container>
        }
        <Container id='controlPanel-controls'>
            { props.controls }
        </Container>
        { window.top.innerWidth < 601 && state.showCode && <MonacoEditor
            options={{
                readOnly: true
            }}
            defaultLanguage="typescript"
            value={props.files ? props.files[0].text : ''}
        />
        }
    </Panel>;
};

export default ControlPanel;
